module PagesCms
  class Page < ActiveRecord::Base
    belongs_to  :parent,    class_name: 'Page'
    has_many    :children,  class_name: 'Page', foreign_key: 'parent_id', dependent: :destroy
    has_many    :blocks, dependent: :destroy

    validates :slug,  presence: true, uniqueness: true
    validates :title, presence: true

    validate :unique_on_blog_and_page

    before_validation :parameterize_slug
    validate  :page_parent_of_itself

    def self.home
      Page.find_by(home: true)
    end

    def parent?
      parent_id.present?
    end

    def self.all_parents
      where('parent_id IS NULL')
    end

    def children?
      children.present?
    end

    private
    def parameterize_slug
      if parent?
        self.slug = "#{parent.slug}/#{title.parameterize}"
      else
        self.slug = title.parameterize
      end
    end

    def page_parent_of_itself
      if self.parent_id != nil && self.parent_id == self.id
        errors.add(:base, 'Cannot be a sub-page of itself')
      end
    end

    def unique_on_blog_and_page
      if Blog.where('title = ? AND id != ?', self.title, self.id).present? or Page.where('title = ? AND id != ?', self.title, self.id).present?
        errors.add(:base, 'Title must be unique.')
      end
    end

  end
end